package de.hs_mannheim.informatik.domain;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayInputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.nio.charset.StandardCharsets;
import java.time.DateTimeException;
import java.time.LocalTime;
import java.time.format.DateTimeFormatter;
import java.time.format.DateTimeFormatterBuilder;
import java.time.temporal.ChronoField;
import java.util.ArrayList;
import java.util.Comparator;

import de.hs_mannheim.informatik.gui.HighscoreMenuGUI;

public class HighscoreManager {
	
	public static void main(String[] args) {
		new HighscoreManager();
	}
	
    public void sortByTime(byte[] data, String filename) throws IOException {
        ArrayList<HighscoreEintrag> highscores = new ArrayList<>();
        	try (BufferedReader reader = new BufferedReader(
                new InputStreamReader(new ByteArrayInputStream(data), StandardCharsets.UTF_8))) {
            
            String line;
            while ((line = reader.readLine()) != null) {
                try {
                    String[] parts = line.split(" ", 2);
                    String timeStr = parts[0];
                    String name = (parts.length > 1) ? parts[1].trim() : "";
                    String[] timeParts = timeStr.split(":");
                    if (timeParts.length != 2) {
                        System.err.println("Invalid time format: " + timeStr);
                        continue;
                    }
                    int hour = Integer.parseInt(timeParts[0]);
                    int minute = Integer.parseInt(timeParts[1]);

                    if (hour < 0 || hour > 23 || minute < 0 || minute > 59) {
                        System.err.println("Invalid time values: " + timeStr);
                        continue;
                    }
                    LocalTime time = LocalTime.of(hour, minute);
                    highscores.add(new HighscoreEintrag(time, name));
                } catch (NumberFormatException | DateTimeException ex) {
                    System.err.println("Error parsing line: " + line);
                    System.err.println("Error details: " + ex.getMessage());
                }
            }
        }
        highscores.sort(Comparator.comparing(HighscoreEintrag::getTime));
        ArrayList<String> formattedHighscores = new ArrayList<>();
        for (HighscoreEintrag e : highscores) {
            String formattedTime = String.format("%02d:%02d", e.getTime().getHour(), e.getTime().getMinute());
            String line = formattedTime + " " + e.getName(); 
            formattedHighscores.add(line);
        }
        String fileName = filename.substring(filename.lastIndexOf('/') + 1);
        File file = new File(fileName);
        overwriteContentOfFile(formattedHighscores, file);
    }
    
	private void overwriteContentOfFile(ArrayList<String> formattedHighscores, File file) throws FileNotFoundException, IOException {
        try (BufferedWriter writer = new BufferedWriter(
                new OutputStreamWriter(new FileOutputStream(file), StandardCharsets.UTF_8))) {	          
            for (String entry : formattedHighscores) {
                writer.write(entry);
                writer.newLine();
            }
        }
    }
	
	public String getAvgTime(byte[] data, String filename) throws IOException {
		int totalSeconds = 0;
		int count = 0;
        	try (BufferedReader reader = new BufferedReader(
	                new InputStreamReader(new ByteArrayInputStream(data), StandardCharsets.UTF_8))) {
                String line;
                DateTimeFormatter dtf = new DateTimeFormatterBuilder()
                	    .appendPattern("mm:ss")                     
                	    .parseDefaulting(ChronoField.HOUR_OF_DAY, 0) 
                	    .toFormatter();
        		while ((line = reader.readLine()) != null) {
                    line = line.trim();
                    if (line.isEmpty()) continue;
                    String[] parts = line.split("\\s+", 2);
                    String timePart = parts[0];
                    LocalTime localTime = LocalTime.parse(timePart, dtf);
                    int minutes = localTime.getMinute(); 
                    int seconds = localTime.getSecond(); 
                    int totalSecThisLine = minutes * 60 + seconds;
                    totalSeconds += totalSecThisLine;
                    count++;
        	}
        	int timeInt =  totalSeconds/count;
        	int minutes = timeInt/60;
        	int seconds = timeInt%60;
        	String timeString= "Durchschnittszeit: " + minutes+":"+seconds;
        	return timeString; 
        	}
	}
	
    public void copyResourceIfNotExists(String resourcePathInJar, File outFile) throws IOException {
        if (outFile.exists()) {
            return; 
        }
        outFile.getParentFile().mkdirs();
        try (InputStream in = HighscoreMenuGUI.class.getResourceAsStream(resourcePathInJar);
             FileOutputStream fos = new FileOutputStream(outFile)) {
            if (in == null) {
                throw new FileNotFoundException("Resource not found in JAR: " + resourcePathInJar);
            }
            byte[] buffer = new byte[4096];
            int bytesRead;
            while ((bytesRead = in.read(buffer)) != -1) {
                fos.write(buffer, 0, bytesRead);
            }
        }
    }
}
